unit DialogDemoMain;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, fdDialog, ExtCtrls, StdCtrls,
  EditBtn;

type

  { TDialogdemo }

  TDialogdemo = class(TForm)
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Button4: TButton;
    Button5: TButton;
    Button6: TButton;
    chkboxMitglied: TCheckBox;
    chkboxZeitraum: TCheckBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label5: TLabel;
    ListeBis: TDateEdit;
    ListeFuer: TLabeledEdit;
    ListeVon: TDateEdit;
    pnlAuswahl: TPanel;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure Button5Click(Sender: TObject);
    procedure Button6Click(Sender: TObject);
    procedure chkboxMitgliedChange(Sender: TObject);
    procedure chkboxZeitraumChange(Sender: TObject);
    function Auswahlok(btn: TdlgResult): boolean;
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
    procedure FormShow(Sender: TObject);
  private

  public

  end;

var
  Dialogdemo: TDialogdemo;

implementation

{$R *.lfm}

var starting: boolean = true;



////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//
// Einfache Ja/Nein Fragen
//
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

procedure TDialogdemo.FormShow(Sender: TObject);
begin
  DlgSetDefaultCancel(dlgYes,dlgNone); // kein Cancel Button
  if not AskYN('Wollen Sie'#13'das Programm'#13'starten?') then close;
  starting:=false;
end;

procedure TDialogdemo.FormCloseQuery(Sender: TObject; var CanClose: boolean);
begin
  if not starting and not AskYN('Programm beenden?') then CanClose:=false;
end;



////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//
// Frage mit umfangreichem Panel und Prüfroutinen
//
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

// Aktivieren der verschiedenen Eingabefelder
procedure TDialogdemo.chkboxMitgliedChange(Sender: TObject);
begin
  if chkboxMitglied.checked
  then ListeFuer.enabled := true
  else begin ListeFuer.enabled := false; ListeFuer.Text := '' end;
end;

procedure TDialogdemo.chkboxZeitraumChange(Sender: TObject);
begin
  if chkboxZeitraum.checked
  then begin ListeVon.enabled := true; ListeBis.enabled := true;
       Label3.Enabled:=true; Label5.Enabled:=true;
       end
  else begin ListeVon.enabled := false; ListeVon.Text := '';
       ListeBis.enabled := false; ListeBis.Text := '';
       Label3.Enabled:=false; Label5.Enabled:=false;
       end;
end;

// Prüfroutine für Dialog von Button 1
function TDialogdemo.Auswahlok(btn: TdlgResult): boolean;
const
  EMgl='Mitgliedsnummern müssen ungerade Zahlen sein';
  EAnf='Ungültiges Anfangsdatum';
  EEnd='Ungültiges Enddatum';
  EZeit='Das Enddatum darf nicht vor dem Anfangsdatum liegen';
var
  von,bis: TDate;
  h: integer;
begin
  Result:=false;
  if chkboxMitglied.Checked and (not TrystrtoInt(ListeFuer.text,h) or not odd(h))
  then begin ErrMsg(EMgl); exit end;
  if chkboxZeitraum.Checked
  then begin
       if not tryStrtoDate(ListeVon.Text,von) then begin Errmsg(EAnf); exit end;
       if not tryStrtoDate(ListeBis.Text,bis) then begin Errmsg(EEnd); exit end;
       if von>bis then begin ErrMsg(EZeit); exit end;
       end;
  Result:=true;
end;


// Aufruf des Dialogs
procedure TDialogdemo.Button1Click(Sender: TObject);
begin
  // Panel pnlAuswahl ins Formular eingehängen
  DlgAddControl(pnlAuswahl);
  // Prüfroutine für die Eingabefelder festlegen
  DlgSetCanClose(@AuswahlOk);
  // Texte für Ja/Nein Buttons
  // Dialogaufruf - zwei Standardbuttons OK und Abbrechen, kein Fragetext
  if AskYN('',dlgOK,dlgCancel)
  then begin
       // 6 Sekunden Timeout
       DlgSetTimeout(6000);
       // Dialogaufruf mit OK Button und Timeout
       msg('Die Liste wurde erfolgreich erstellt',[dlgOK]);
       end;
end;


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//
// Frage mit individuellen Knöpfen
//
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

procedure TDialogdemo.Button2Click(Sender: TObject);
const
  Meldungen: array[dlgBtn1..dlgBtn4] of string =
    ('Puh, das Springen war anstrengend!',
     'Ah, das Laufen hat Spaß gemacht!',
     'Ohh, die Koffer waren vielleicht schwer!',
     'Mmmm, der Whisky war ausgezeichnet!');
var
  res: TdlgResult;
begin
  repeat
      // 4 individuelle Buttons
      dlgAddButtons('Springen', 'Laufen', 'Koffer tragen', 'Whisky trinken');
      dlgSetposition(0,0);
      // und zusätzlich ein Standardbnutton
      res:=Ask('Was möchten Sie tun?',[dlgAbort]);
      if res=dlgAbort then break;

      dlgSetTimeout(3000);      // 3 Sekunden Timeout
      dlgSetDlgType(dlgNoIcon); // Kein Icon beim Dialog
      msg (Meldungen[res]);
      until not AskYN('Möchten Sie weitermachen?');
end;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//
// Frage in Verbindung mit Warnungsicon
//
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

procedure TDialogdemo.Button3Click(Sender: TObject);
begin
  dlgSetDlgType(dlgWarn);
  dlgSetDefaultCancel(dlgNo, dlgNo);
  if AskYN('Vorsicht! Was Sie da machen,'#13'das ist brandgefährlich'#13#13
         +'Sind Sie sicher, dass Sie das wollen?')
  then msg('Puh, Glück gehabt!'#13'Das ist gerade noch gut gegangen!')
  else msg('Sie haben recht, das war wahrscheinlich besser so');
end;

procedure TDialogdemo.Button4Click(Sender: TObject);
begin
  dlgSetposition(0,0,false);
  dlgAddButtons('Einschalten','Ausschalten','unverändert belassen');
  case Ask('Sind Sie sicher, dass Sie für die restliche Laufzeit'#13
          +'des Programms die Anzeige der Icons umschalten wollen?'#13#13
          +'Hinweis: Das betrifft nur Dialoge, bei denen nicht'#13
          +'manuell ein Icontyp gewählt wurde') of
    dlgBtn1: begin dlgSetStandardIcons(true);
             msg('Die Iconanzeige wurde eingeschaltet');
             end;
    dlgBtn2: begin dlgSetStandardIcons(false);
             msg('Die Iconanzeige wurde ausgeschaltet');
             end;
    dlgBtn3: msg('Die Iconanzeige bleibt unverändert');
    end (* case *);

end;

procedure TDialogdemo.Button5Click(Sender: TObject);
begin
  repeat dlgSetTimeout(3000);
      dlgSetFontsize(8);
      dlgAddButtons('sofort positiv entscheiden','lieber nicht gleich entscheiden');
      case ask('Möchten Sie, unter Berücksichtigung der entscheidenen und der unwesentlichen Faktoren,'#13
              +'die Ihnen zur Kenntnis gebracht worden sind, lieber sofort eine positive Entscheidung'#13
              +'treffen oder möchten Sie diese Entscheidung lieber zurückstellen, bis Sie genauere'#13
              +'Informationen aus dem Internet abgerufen haben, obwohl Ihnen natürlich klar sein muss,'#13
              +'dass alle Informationen, die Sie im Internet finden können, im höchten Grad unzuverlässig'#13
              +'sind?') of
        dlgBtn1: begin msg('Sehr gut! Das ist tapfer!'); break end;
        dlgBtn2: msg('Zaudern Sie doch nicht so herum!');
        dlgTimeOut: errmsg('Machen Sie endlich Ihre Auswahl, Sie Schlafmütze!');
        end (* case *);
      until false;
end;

procedure TDialogdemo.Button6Click(Sender: TObject);
var
  res: TdlgResult;
  nr: string;
begin
  repeat DlgSetDefaultCancel(dlgBtn4, dlgBtn6);
      DlgAddButtons('B1','B2','B3','B4','B5');
      DlgAddButtons('B6','B7','B8','B9','B10');
      DlgAddControl(Listevon); Listevon.Enabled:=true;
      Res:=Ask('Bitte geben Sie ein Datum ein und drücken Sie anschließend einen Button?',[dlgAbort]);
      // if Res=dlgAbort then exit;
      nr:=inttostr(ord(res)-ord(LastStandardButton));
      until not AskYN('Sie haben Button Nr. '+nr+' angeklickt'#13#13
                     +'Wollen Sie weitermachen?');
end;


end.

