unit fdDialog;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//
//  fdDialog 1.1
//  Release  12.Dez.2020
//  Autor    Franz Müller
//
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

{$ifdef windows}
{$define win}
{$endif}


interface

uses
  LCLIntf, LCLType, SysUtils, Classes, Graphics,
  {$ifdef win} windows, {$endif}
  Controls, Forms, StdCtrls, Buttons, Math, ExtCtrls;

type
  TDlgResult=(// No Button
              dlgNone,dlgTimeOut,
              // Standard "cancel" buttons
              dlgCancel,dlgAbort,dlgNo,dlgClose,
              // Standard "default" buttons
              dlgOK,dlgYes,dlgRetry,dlgOpen,dlgSave,
              // Other standard buttons
              dlgNoToAll,dlgYesToAll,dlgHelp,dlgIgnore,
              // Individual Buttons
              dlgBtn1,dlgBtn2,dlgBtn3,dlgBtn4,dlgBtn5,
              dlgBtn6,dlgBtn7,dlgBtn8,dlgBtn9,dlgBtn10);

const
  DlgYN=[dlgNo,dlgYes];
  DlgYNAll=[dlgNo,dlgNoToAll,dlgYes,dlgYesToAll];
  DlgAR=[dlgAbort,dlgRetry];
  DlgRI=[dlgRetry,dlgIgnore];
  DlgARI=[dlgAbort,dlgRetry,dlgIgnore];

  FirstStandardButton=dlgCancel;
  LastStandardButton=dlgIgnore;
  
type
  TDlgButton=FirstStandardButton..LastStandardButton; 
  TDlgButtonset=set of TdlgButton;

  TDlgType=(dlgDefault, dlgNoIcon, dlgInfo, dlgWarn, dlgError, dlgQuestion);

  TCanCloseFunc = function(Btn: TDlgResult): boolean of object;

  procedure DlgSetStandardFont (Font: TFont);
  procedure DlgSetStandardFontSize (Fontsize: integer);
  procedure DlgSetStandardIcons (ShowIcon: boolean);
  procedure DlgSetStandardText (DlgButton: TDlgButton; Const Txt: string);
  procedure DlgSetStandardColors (info,warn,error,question: TColor);
  procedure DlgSetStandardSpaces (Border, Top, Bottom, HDistance, VDistance: integer);

  procedure DlgReset;
  procedure DlgSetposition (left,top: integer; relative: boolean = true);
  procedure DlgSetFont (Font: TFont);
  procedure DlgSetFontsize (size: integer);
  procedure DlgSetDlgType( DlgType: TDlgType);
  procedure DlgSetTimeout (ms: integer);
  procedure DlgSetCanClose (Canclose: TCanCloseFunc);
  procedure DlgAddControl (co: TWinControl);
  procedure DlgAddButtons (const Btn1: string; const Btn2: string='';
                           const Btn3: string=''; const Btn4: string='';
                           const Btn5: string='');
  procedure DlgSetDefaultCancel (Default, Cancel: TDlgResult);

  function Msg (const txt: string; Btns: TDlgButtonSet = []): TDlgResult;
  function ErrMsg (const txt: string; Btns: TDlgButtonSet = []): TDlgResult;
  function WarnMsg (const txt: string; Btns: TDlgButtonSet = []): TDlgResult;
  function Ask (const txt: string; Btns: TDlgButtonSet = []): TDlgResult;
  function AskYN (const txt: string): boolean; overload;
  function AskYN (const txt: string; Yes,No: TDlgResult): boolean; overload;

type

TDialog = class(TForm)
  Pic: TImage;
  lblFrage: TLabel;
  // pnlButtons: TPanel;
  dlgTimer: TTimer;
  procedure BitBtnClick(Sender: TObject);
  procedure dlgTimerTimer(Sender: TObject);
  procedure FormPaint(Sender: TObject);
  procedure FormShow(Sender: TObject);
private
  DlgType: TDlgType;
  ManualPosition: boolean;
  RelativePosition: boolean;
  TimeOut: integer;
  CanClose: TCanCloseFunc;
  CustomComponent: TWinControl;
  CCParent: TWinControl;

  BtnCount: integer;
  BtnDefault, BtnCancel: TDlgResult;
  LastAddedButton: TdlgResult;
  FirstButton,DefaultBtn: TBitBtn;
  dlgResult: TdlgResult;
  FirsttimePaint: boolean;
  function doDlg (question: string): TdlgResult;
public
  constructor create (AOwner: TComponent); override;
  procedure CreateParams(var Params: TCreateParams); override;
  {$Ifdef WIN}
  procedure WMNCHitTest(var Message: TWMNCHitTest); message WM_NCHITTEST;
  {$endif}
end;

var
dlg: TDialog;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//
implementation
//
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

{$R *.lfm}

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//
//  Globale Initialisierung, Defaultwerte
//
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

var

MustCreateDlg: boolean=true;
FormCount: integer = 0;

HDist: integer=20;
VDist: integer=30;
MinBorder: integer=30;
TopMargin: integer=30;
Bottommargin: integer=20;
StandardIcons: boolean = true;
StandardFont: TFont = nil;
StandardFontsize: integer = 14;
Standardtexte: array[TDlgButton] of string =
  ('Abbrechen','Abbrechen','Nein','&Schließen','OK','&Ja','',
   'Öffnen','Speichern','Alle Nein','Alle Ja','Hilfe','&Ignorieren');

// ('Info','Achtung!','Fehler!','Frage');
Titeltexte: array[dlgInfo..dlgQuestion] of string =
  ('','','','');
StandardFarben: array [dlgDefault..dlgQuestion] of TColor =
  (clDkGray,clDkGray,clDkGray,clDkGray,clred,clDkGray);

procedure DlgSetStandardFont(Font: TFont);
begin
  if assigned(font) then Standardfont:=Font;
end;

procedure DlgSetStandardFontsize (FontSize: integer);
begin
  StandardFontsize:=FontSize;
end;

procedure DlgSetStandardIcons(ShowIcon: boolean);
begin
  StandardIcons:=ShowIcon;
end;


procedure DlgSetStandardText (DlgButton: TDlgButton; Const Txt: string);
begin
  StandardTexte[DlgButton]:=Txt;
end;


procedure DlgSetStandardColors (info,warn,error,question: TColor);
begin

  StandardFarben[dlgDefault]:=info;
  StandardFarben[dlgNoIcon]:=info;
  StandardFarben[dlgInfo]:=info;
  StandardFarben[dlgWarn]:=warn;
  StandardFarben[dlgError]:=error;
  StandardFarben[dlgQuestion]:=question;
end;

procedure DlgSetStandardSpaces(Border, Top, Bottom, HDistance, VDistance: integer);
begin
HDist:=HDistance;
VDist:=VDistance;
MinBorder:=Border;
TopMargin:=Top;
Bottommargin:=Bottom;
end;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//
//  Create / Destroy
//
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

procedure TDialog.CreateParams(var Params: TCreateParams);
begin
inherited;
Params.Style := Params.Style or WS_POPUP or WS_THICKFRAME;
Params.ExStyle := WS_EX_TOOLWINDOW;
end;

{$Ifdef WIN}
procedure TDialog.WMNCHitTest(var Message: TWMNCHitTest);
begin
  inherited;
  Message.Result := htCaption;
end;
{$endif}


Constructor TDialog.Create(AOwner: TComponent);
begin
  inherited;
(*  SetWindowLong(Handle, GWL_STYLE,
                GetWindowLong(Handle, GWL_STYLE) and not(WS_DLGFRAME)); *)
  Caption:=#0;  // muss noch gesetzt werden
  if assigned(StandardFont) then Font:=Standardfont;
  Font.Size:=StandardFontsize;
  if StandardIcons then DlgType:=dlgDefault else DlgType:=dlgNoIcon;
  dlg.BtnDefault:=dlgTimeOut; // Default Button noch nicht gesetzt
  dlg.BtnCancel:=dlgTimeOut; // Cancel Button noch nicht gesetzt
  FirstTimePaint:=true;
  LastAddedButton:=LastStandardButton; // Vor dem ersten individuellem Button
end;

procedure DlgReset;
begin
  if Dlg=nil then exit;
  Dlg.dlgTimer.Enabled:=false;
  if Dlg.CustomComponent<>nil
  then begin dlg.CustomComponent.Visible:=false;
       dlg.Customcomponent.parent:=Dlg.CCParent;
       dlg.Customcomponent:=nil;
       end;
  Freeandnil(Dlg);
  MustCreateDlg:=true;
end;



///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//
//  Dialogparameter ändern
//
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

procedure CheckDialog;
begin
  if MustCreateDlg
  then begin Application.Createform(TDialog, dlg); MustCreateDlg:=false end;
end;

procedure DlgSetposition(left,top: integer; relative: boolean = true);
begin
  CheckDialog;
  dlg.RelativePosition:=relative;
  dlg.left:=left; dlg.top:=top;
  dlg.left:=left; dlg.top:=top;
  dlg.ManualPosition:=true;
end;

procedure DlgSetFont(Font: TFont);
begin
  CheckDialog;
  dlg.Font:=Font;
end;

procedure DlgSetFontsize(size: integer);
begin
  CheckDialog;
  dlg.Font.Size:= size;
end;

procedure DlgSetDlgType(DlgType: TDlgType);
begin
  CheckDialog;
  dlg.dlgtype:=DlgType;
end;

procedure DlgSettimeout (ms: integer);
begin
  CheckDialog;
  dlg.TimeOut:=ms;
end;

procedure DlgSetCanClose (Canclose: TCanCloseFunc);
begin
  CheckDialog;
  dlg.CanClose:=CanClose;
end;

procedure DlgAddControl (co: TWinControl);
begin
  CheckDialog;
  dlg.CustomComponent:= co;
  co.TabOrder:=0;
end;

procedure DlgSetDefaultCancel (Default, Cancel: TDlgResult);
begin
  CheckDialog;
  dlg.BtnDefault:=Default; dlg.BtnCancel:=Cancel;
end;

procedure DlgAddButton (Btn: string; btnResult:TDlgResult);
var
  B:TBitBtn;

  procedure MaybeSetDefault;
  begin
  if dlg.Btndefault=dlgTimeOut // Default noch nicht gesetzt
  then dlg.BtnDefault:=btnResult;
  end;
  procedure MaybeSetCancel;
  begin
  if dlg.Btncancel=dlgTimeOut  // Cancel noch nicht gesetzt
  then dlg.Btncancel:=btnResult;
  end;
    
begin
  B:=TBitBtn.Create(dlg); //.pnlButtons);
  B.Parent:=dlg; //.pnlButtons;
  inc(dlg.BtnCount);
  B.Autosize:=true;
  B.Font:=dlg.Font;
  B.font.Size:=dlg.Font.Size;
  B.Caption:=Btn;
  B.OnClick:=@dlg.BitBtnClick;
  B.Tag:=ord(btnResult);
  if dlg.FirstButton=nil then dlg.FirstButton:=B;
  case btnResult of
    dlgCancel..dlgClose: MaybeSetCancel;
    dlgOK..dlgSave: MaybeSetDefault;
    end (* case *);
  if btnResult=dlg.BtnDefault then begin B.Default:=true; dlg.DefaultBtn:=B end;
  if btnResult=dlg.BtnCancel then B.Cancel:=true;
end;


procedure DlgAddButtons (const Btn1: string; const Btn2: string='';
                           const Btn3: string=''; const Btn4: string='';
                           const Btn5: string='');

  procedure DlgAddIndividualButton (Btn: string); inline;
  const
    TooMany='In einem Dialog sind nicht mehr als 10 individuelle Buttons möglich';
  begin
    if Btn='' then exit;
    if dlg.LastAddedButton=High(TDlgResult)
    then raise Exception.Create(TooMany);
    inc(dlg.LastAddedButton);
    DlgAddButton(Btn, dlg.LastAddedButton);
  end;

begin
  CheckDialog;
  DlgAddIndividualButton(Btn1);
  DlgAddIndividualButton(Btn2);
  DlgAddIndividualButton(Btn3);
  DlgAddIndividualButton(Btn4);
  DlgAddIndividualButton(Btn5);
end;

Procedure DlgSetStdButtons (Btns: TDlgButtonset);
var
  b: TDlgButton;
begin
  for b in Btns do
     DlgAddButton (standardtexte[b], b);
  if (dlg.BtnCount=0) and (dlg.TimeOut=0)
  then DlgAddButton (standardtexte[dlgOK], dlgOK);
end;

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//
//  Ereignisbehandlung:
//  FormShow
//  BitBtnClick
//  TimerTimer
//
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////



procedure TDialog.BitBtnClick(Sender: TObject);
begin
dlgResult:=TDlgResult(TBitBtn(Sender).Tag);
if (dlgResult in [dlgAbort,dlgCancel]) // Cancel, abort darf immer schließen
or not assigned(Canclose)              // keine Prüfroutine definiert
or Canclose(dlgResult)                 // Prüfroutine war erfolgreich
then begin dlgTimer.Enabled:=false;
     ModalResult:=mrOK;    // beliebig, richtiges Ergebnis steht in dlgResult
     end;
end;


procedure TDialog.dlgTimerTimer(Sender: TObject);
begin
dlgTimer.Enabled:=false;
dlgResult:=dlgTimeOut;
ModalResult:=mrOK;   // beliebig, richtiges Ergebnis steht in dlgResult
end;



////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//
//  Dlg
//  Formshow
//  formPaint
//
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

function TDialog.doDlg (question: string): TdlgResult;

    procedure SetDlgIcon;
    (*$ifdef win*)
    const
      DialogIcons: array[dlgInfo..dlgQuestion] of
        {$ifdef UNICODE} PWIDECHAR
        {$else}          PANSICHAR
        {$endif}
       = (IDI_ASTERISK, IDI_EXCLAMATION, IDI_HAND,  IDI_QUESTION);
    begin
      if DlgType>=dlgInfo
        then begin Pic.Visible:=true;
             Pic.Picture.Icon.Handle:=loadicon(0,DialogIcons[DlgType]);
             end
        else Pic.Visible:=false;
    end (* SetIcon *);
    (*$else*)
    begin
      Pic.Visible:=false
    end;
    (*$endif*)


begin
  inc(FormCount);
  try
    if question<>''
    then begin SetDlgIcon;
         lblFrage.Caption:=question;
         lblFrage.Font.Size:=Font.Size;
         lblFrage.Visible:=true;
         end
    else begin Pic.Visible:=false;
         lblFrage.Visible:=false;
         end;

    if assigned(CustomComponent)
    then begin customcomponent.visible:=true; CustomComponent.Taborder:=0;
         CCParent := CustomComponent.Parent; CustomComponent.Parent := self;
         end;

    MustCreateDlg:=true; // Ab jetzt ist Dialogaufruf rekursiv
    ShowModal;
    Result := DlgResult;
  finally
    dec(FormCount);
    Dlg:=self; // Falls im Showmodal ein neuer Dialog gestartet wurde
    DlgReset;
  end;

end;


procedure TDialog.FormShow(Sender: TObject);
var
  DataHeight, DataWidth: integer;
  qw, bw, bh, d, l, h, i, t, w: integer;
  HasCustomComponent: boolean;
  Caller: TForm;

begin // Die Position und Größe der Controls berechnen
Caller:=Screen.ActiveForm;
DataHeight:=Topmargin;

// lblFrage und Pic - Platzbedarf  & vertikale. Position
if lblFrage.Visible
then begin lblFrage.Top:=DataHeight;
     Pic.Top:=lblFrage.Top + (lblFrage.Height-Pic.Height) div 2;
     inc(DataHeight,lblFrage.Height+Vdist);
     qw:=lblFrage.Width+ord(Pic.Visible)*50;
     end
else qw:=50;

// CustomComponents Platzbedarf & vertikale Position
HasCustomComponent:=CustomComponent<>nil;
if HasCustomComponent
then begin DataWidth:=max(qw,Customcomponent.Width);
     Customcomponent.Top:=DataHeight;
     inc(DataHeight, CustomComponent.Height+VDist);
     end
else DataWidth:=qw;

// Buttons - Platzbedarf, Default- und Cancel Property
bw:=0; bh:=0;
if BtnCount>0
then begin
     for i:=0 to ControlCount-1 do if Controls[i] is TBitBtn
        then with TBitBtn(Controls[i]) do
          begin 
          inc(bw,width); bh:=max(bh,height);  // Width und Height vom Button
          Top:=DataHeight;
          if ord(BtnDefault)=Tag
          then begin Default:=true;
               Taborder:= ord(HasCustomComponent);
               end;
          if ord(BtnCancel)=Tag then Cancel:=true;
          end (* for with *);
     DataWidth:=max(DataWidth,bw+(BtnCount-1)*HDist);
     inc(DataHeight, bh);
     end;

// lblFrage und Pic - Position
h:=MinBorder+(Datawidth-qw) div 2;
if Pic.Visible
then begin Pic.Left:=h; lblFrage.Left:=h+50 end
else lblFrage.Left:=h;

// CustomComponents - horizontale Position
if CustomComponent<>nil
then customcomponent.Left:=(Datawidth-customcomponent.width) div 2 + MinBorder;

// Buttons - Position
case BtnCount of
  0: (* Nichts zu berechnen *) ;
  1: with TBitBtn(Controls[ControlCount-1]) do (* letztes Control *)
       begin
       left := (DataWidth*2) div 3;
       inc(DataHeight,Height);
       end;
  else begin
       d:=(DataWidth+2*MinBorder-bw) div (Btncount+1);
       l:=d; h:=0;
       for i:=0 to ControlCount-1 do
           if Controls[i] is TBitBtn
           then with TBitBtn(Controls[i]) do
                begin Left:=l; inc(l,Width+d); h:=max(h,Height);
                //debug('%s: Taborder=%d',[Caption,Taborder]
                end;
       inc(DataHeight,h);
       end;
  end; (* case *)

// Form
Width:=DataWidth+2*MinBorder;
Height:=DataHeight+Bottommargin;
if assigned(caller)
then begin l:=caller.left; w:=caller.width; t:=caller.Top; h:=caller.Height end
else begin l:=0; w:=screen.width; t:=0; h:=screen.height end;
if ManualPosition
then begin
     if RelativePosition then begin left:=left+l; Top:=Top+t end
     end
else begin
     Left := l + (w-Width) div 2 + 120*(FormCount-1);
     Top := t + (h-Height) div 2 + 100*(FormCount-1)
     end;
if screen.monitorcount=1 // Positionskorrektur nur mit einem Monitor
then begin Top:=min(Top, Screen.Height-Height);
     Left:=min(Left, Screen.Width-Width);
     end;

// Start Timeout timer
if TimeOut>0
then begin dlgTimer.Interval:=TimeOut; dlgTimer.enabled:=true end;
end;


procedure TDialog.FormPaint(Sender: TObject);
var s: string;
begin
  Canvas.pen.color := Standardfarben[dlgType];
  Canvas.pen.width := 2;
  Canvas.RoundRect(1,1,width-1,height-1,20,20);
  if FirsttimePaint
  then begin
       FirstTimePaint:=false;
       if (DefaultBtn<>nil) and (ActiveControl=FirstButton)
       then ActiveControl:=DefaultBtn;
       end;
end;


////////////////////////////////////////////////////////////////////////////////
//
//  Dialog-Aufrufe
//
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

procedure MaybeSetDialogType (t: TdlgType);
begin
// Set dialog type if it has not been set manually
if dlg.dlgType=dlgDefault then dlg.dlgType:=t;

if dlg.caption=#0 then
   if dlg.dlgtype>dlgNoIcon
   then dlg.Caption:=Titeltexte[dlg.dlgtype]
   else dlg.Caption:=Titeltexte[t]
end;

function xDlg(const txt: string; Btns: TDlgButtonSet; dlgType: TdlgType): TdlgResult;
begin
  CheckDialog;
  MaybeSetDialogType(dlgType);
  DlgSetStdButtons(Btns);
  result:=dlg.doDlg(txt);
end;

function Msg (const txt: string; Btns: TDlgButtonSet = []): TDlgResult;
begin
  result:=xDlg(txt, Btns, dlgInfo);
end;

function ErrMsg (const txt: string; Btns: TDlgButtonSet = []): TDlgResult;
begin
  result:=xDlg(txt, Btns, dlgError);
end;

function WarnMsg (const txt: string; Btns: TDlgButtonSet = []): TDlgResult;
begin
  result:=xDlg(txt, Btns, dlgWarn);
end;

function Ask (const txt: string; Btns: TDlgButtonSet = []): TDlgResult;
begin
  result:=xDlg(txt, Btns, dlgQuestion);
end;

function AskYN (const txt: string): boolean;
var
  c: integer;
  s: TDlgButtonset;
  yes: TDlgresult;
begin
  if MustCreateDlg then c:=0
                   else c:=Dlg.BtnCount;
  case c of
    0:  begin s:=dlgYN; Yes:=dlgYes end;
    2:  begin s:=[]; Yes:=dlgBtn1 end;
    else raise exception.Create('Ungültige Buttonzahl in AskYN');
    end (* case *);
  result:=xDlg(txt, s, dlgQuestion)=Yes;
end;

function AskYN (const txt: string; Yes,No: TDlgResult): boolean;
var
  s: TDlgButtonset;
  c: integer;

  procedure MaybeAddBtn(b: TDlgResult);
  begin
    // Standardbuttons werden hier erzeugt,
    // individuelle Buttons sind hier schon erzeugt worden
    if b in [FirstStandardButton..LastStandardButton]
    then begin include(s,b); inc(c) end;
  end;

begin
  if yes=no then raise exception.Create
                 ('Yes und No müssen verschiedene Buttons sein');
  if MustCreateDlg then c:=0
                   else c:=Dlg.BtnCount;
  s:=[]; MaybeAddBtn(Yes); MaybeAddBtn(No);
  if c<>2 then raise exception.Create('Ungültige Buttonzahl in AskYN');
  result:=xDlg(txt, s, dlgQuestion)=Yes;
end;



initialization
Standardfont:=nil
end.






